{% extends "account/base.html" %}

{% load i18n %}

{% block head_title %}{% trans "Delete Password" %}{% endblock %}

{% block content %}
    <h1>{% trans "Delete Password" %}</h1>
    <p>{% blocktrans %}You may delete your password since you are currently logged in using OpenID.{% endblocktrans %}</p>
    <form method="post" action="">
        {% csrf_token %}
        <input type="submit" value="{% trans "delete my password" %}">
    </form>
{% endblock %}
