{% extends "account/base.html" %}

{% load i18n %}

{% block head_title %}{% trans "Password Deleted" %}{% endblock %}

{% block content %}
    <h1>{% trans "Password Deleted" %}</h1>
    <p>{% blocktrans %}Your password has been deleted.{% endblocktrans %}</p>
{% endblock %}
